//1E Disco Dance Floor
//Firmware Revision 2.0
//Grant Elliott

#define VERSION 0x20

#include <ioavr.h>
#include <inavr.h>
#include "TWI_Master.c"
#include "usart.c"

//Masks
#define RED_MASK 0x00
#define GREEN_MASK 0x01
#define BLUE_MASK 0x08
#define SENSOR_MASK 0x09
const unsigned char ROW_MASKS[]={0x20, 0x22, 0x10, 0x12};

//Error Codes
#define CODE_SUCCESS 0x00
#define CODE_BADCOMMAND 0x01
#define CODE_BADARGS 0x02

//function declarations

void ftdi_init();
void send_status_byte(unsigned char status);

unsigned char module_init();
unsigned char chip_init(unsigned char slave);
unsigned char sensor_init(unsigned char slave);

unsigned char power_down();
unsigned char chip_write_blank(unsigned char slave);
unsigned char self_test();

unsigned char module_write();
unsigned char row_write(unsigned char row);
unsigned char chip_write(unsigned char slave);

unsigned char module_read(unsigned char *buffer);
unsigned char row_read(unsigned char row, unsigned char *buffer);

void main()
{
 unsigned char command,temp=CODE_SUCCESS,row;
 unsigned char buffer[8];

 ftdi_init();
 __enable_interrupt();
 usart_init(8);
 TWI_Master_Initialise();
 module_init();
 while(1)
 {
  command=usart_getc();
  switch(command)
  {
   case 16: //0x10
     if (usart_wait_for_data(96))
       send_status_byte(module_write());
     else
       send_status_byte(CODE_BADARGS);
   break;
   
   case 17: //0x11
     if (usart_wait_for_data(25))
       send_status_byte(row_write(usart_getc()));
     else
       send_status_byte(CODE_BADARGS);
   break;
   
   case 32: //0x20
     temp=module_read(buffer);
     send_status_byte(temp);
     if (temp==CODE_SUCCESS)
     {
       usart_putc(buffer[0]);
       usart_putc(buffer[1]);
       usart_putc(buffer[2]);
       usart_putc(buffer[3]);
       usart_putc(buffer[4]);
       usart_putc(buffer[5]);
       usart_putc(buffer[6]);
       usart_putc(buffer[7]);
     }
   break;
   
   case 33: //0x21
     if (usart_wait_for_data(1))
     {
       temp=row_read(usart_getc(),buffer);
       send_status_byte(temp);
       if (temp==CODE_SUCCESS)
       {
         usart_putc(buffer[0]);
         usart_putc(buffer[1]);
       }
     }    
     else
       send_status_byte(CODE_BADARGS);
   break;   
   
   case 48: //0x30
     if (usart_wait_for_data(96))
     {
       temp=module_write();
       temp|=module_read(buffer);
       send_status_byte(temp);
       if (temp==CODE_SUCCESS)
       {
         usart_putc(buffer[0]);
         usart_putc(buffer[1]);
         usart_putc(buffer[2]);
         usart_putc(buffer[3]);
         usart_putc(buffer[4]);
         usart_putc(buffer[5]);
         usart_putc(buffer[6]);
         usart_putc(buffer[7]);
       }
     }    
     else
       send_status_byte(CODE_BADARGS);
   break;
   
   case 49: //0x31
     if (usart_wait_for_data(25))
     {
       row=usart_getc();
       temp=row_write(row);
       temp=row_read(row,buffer);
       send_status_byte(temp);
       if (temp==CODE_SUCCESS)
       {
         usart_putc(buffer[0]);
         usart_putc(buffer[1]);
       }
     }    
     else
       send_status_byte(CODE_BADARGS);
   break;
   
   //Power Down
   case 64: //0x40
     send_status_byte(power_down());
   break;
   
   //Test Repsponse
   case 80: //0x50
     send_status_byte(CODE_SUCCESS);
   break;
   
   //Reset
   case 96: //0x60
     send_status_byte(module_init());
   break;
   
   //Version Number
   case 112: //0x70
     send_status_byte(VERSION);
   break;
   
   //Self-Test
   case 128: //0x80:
     send_status_byte(self_test());
   break;
     
   default:
     send_status_byte(CODE_BADCOMMAND);
   break; 
  }
  usart_flushRx();
 }
}

void ftdi_init()
{
  DDRD=0x0A;         //00001010
  DDRC=0x04;         //00000100
  PORTD=PORTD&0xF7;  //11110111
}

unsigned char module_init()
{
 unsigned char i, code=CODE_SUCCESS;
 for (i=0;i<4;i++)
 { 
  code |= chip_init(ROW_MASKS[i]|RED_MASK);
  code |= chip_init(ROW_MASKS[i]|GREEN_MASK);
  code |= chip_init(ROW_MASKS[i]|BLUE_MASK);
  code |= sensor_init(ROW_MASKS[i]|SENSOR_MASK);
 }
 return code;
}

unsigned char sensor_init(unsigned char slave)
{
  //add error checking

  unsigned char messageBuf[10];
  unsigned char code=CODE_SUCCESS;

  messageBuf[0] = (slave<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
  
  //config register
  messageBuf[1] = 0x0F; 
  messageBuf[2] = 0x00; 
  TWI_Start_Transceiver_With_Data( messageBuf, 3 );

  //port config
  messageBuf[1] = 0x06; 
  messageBuf[2] = 0xFF;
  messageBuf[3] = 0xFF;
  TWI_Start_Transceiver_With_Data( messageBuf, 4 ); 

  return code;
}

unsigned char chip_init(unsigned char slave)
{
  //add error checking
  unsigned char code=CODE_SUCCESS;
  unsigned char messageBuf[10];

  //config register
  messageBuf[0] = (slave<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
  messageBuf[1] = 0x0F; 
  messageBuf[2] = 0x08; 
  TWI_Start_Transceiver_With_Data( messageBuf, 3 );

  //port config
  messageBuf[1] = 0x06; 
  messageBuf[2] = 0x00;
  messageBuf[3] = 0x00;
  TWI_Start_Transceiver_With_Data( messageBuf, 4 ); 
  
  //blink phase 0
  messageBuf[1] = 0x02; 
  messageBuf[2] = 0xFF;
  messageBuf[3] = 0xFF;
  TWI_Start_Transceiver_With_Data( messageBuf, 4 );

  //master intensity full on
  messageBuf[1] = 0x0E;
  messageBuf[2] = 0xFF;
  TWI_Start_Transceiver_With_Data( messageBuf, 3 ); 

  //individual intensities off
  messageBuf[1] = 0x10;
  messageBuf[2] = 0xFF;
  messageBuf[3] = 0xFF;
  messageBuf[4] = 0xFF;
  messageBuf[5] = 0xFF;
  messageBuf[6] = 0xFF;
  messageBuf[7] = 0xFF;
  messageBuf[8] = 0xFF;
  messageBuf[9] = 0xFF;
  TWI_Start_Transceiver_With_Data( messageBuf, 10 ); 

  return code;
}

unsigned char power_down()
{
 unsigned char i,code=CODE_SUCCESS;
 for (i=0;i<4;i++)
 {
  code |= chip_write_blank(ROW_MASKS[i]|RED_MASK);
  code |= chip_write_blank(ROW_MASKS[i]|GREEN_MASK);
  code |= chip_write_blank(ROW_MASKS[i]|BLUE_MASK);
 }
 return code; 
}

unsigned char self_test()
{
 unsigned char code=CODE_SUCCESS;
 //write some code here




 return code;
}

unsigned char module_write()
{
 unsigned char code;
 
 code = row_write(0);
 code |= row_write(1);
 code |= row_write(2);
 code |= row_write(3);

 return code;
}

unsigned char row_write(unsigned char row)
{
 unsigned char code;

 code = chip_write(RED_MASK|ROW_MASKS[row]);
 code |= chip_write(GREEN_MASK|ROW_MASKS[row]);
 code |= chip_write(BLUE_MASK|ROW_MASKS[row]);
 
 return code;
}

unsigned char chip_write(unsigned char slave)
{
  //add error testing

  unsigned char code=CODE_SUCCESS;
  unsigned char messageBuf[10];
  
  messageBuf[0] = (slave<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
  messageBuf[1] = 0x10;
  messageBuf[2] = usart_getc();
  messageBuf[3] = usart_getc();
  messageBuf[4] = usart_getc();
  messageBuf[5] = usart_getc();
  messageBuf[6] = usart_getc();
  messageBuf[7] = usart_getc();
  messageBuf[8] = usart_getc();
  messageBuf[9] = usart_getc();
  TWI_Start_Transceiver_With_Data( messageBuf, 10 ); 

  return code;
}

unsigned char chip_write_blank(unsigned char slave)
{
  //add error testing

  unsigned char code=CODE_SUCCESS;
  unsigned char messageBuf[10];
  messageBuf[0] = (slave<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
  messageBuf[1] = 0x10;
  messageBuf[2] = 0xFF;
  messageBuf[3] = 0xFF;
  messageBuf[4] = 0xFF;
  messageBuf[5] = 0xFF;
  messageBuf[6] = 0xFF;
  messageBuf[7] = 0xFF;
  messageBuf[8] = 0xFF;
  messageBuf[9] = 0xFF;
  TWI_Start_Transceiver_With_Data( messageBuf, 10 ); 
 
  return code; 
}

unsigned char module_read(unsigned char *buffer)
{
 unsigned char code;
 
 code = row_read(0,buffer);
 code |= row_read(1,buffer+2);
 code |= row_read(2,buffer+4);
 code |= row_read(3,buffer+6);

 return code;
}

unsigned char row_read(unsigned char row, unsigned char *buffer)
{
 //add error codes
 unsigned char code=CODE_SUCCESS, slave=ROW_MASKS[row]|SENSOR_MASK;
 unsigned char messageBuf[4];
 messageBuf[0] = (slave<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
 messageBuf[1] = 0x00;
 TWI_Start_Transceiver_With_Data(messageBuf, 2);
 messageBuf[0] = (slave<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT);
 TWI_Start_Transceiver_With_Data(messageBuf, 3);
 TWI_Get_Data_From_Transceiver(messageBuf, 3);
 *buffer=messageBuf[1];
 buffer++;
 *buffer=messageBuf[2];
 return code;
}

void send_status_byte(unsigned char status)
{
  if (usart_unsent_data>0)
    usart_flushTx();
  usart_putc(status);
}
